<?php

    if ( !defined( 'ABSPATH' ) ) exit;
    
    class JGalleryFromFiles595 extends stdClass
    {
        public $image_html      = '';
        private $options         = null;
        private $template_folder = '';
        private $gallery_html    = '';
        private $include_images  = false;
        private $include_folders = false;
        private $errorMessage    = array();
        private $_image = null;
        private $_counter = null;
        private $is_writeable = false;
        
        public function __construct($options) {
           
            $this->options = $options;
            $this->options['token'] = $this->getToken();

            $this->options['folder'] = str_replace('/', DS, $this->options['folder']);
            
            if (isset($this->options['image'])) {
                $this->_image = $this->options['image'];
            }
            if (isset($this->options['counter'])) {
                $this->_counter = $this->options['counter'];
            }

            $validTemplate = false;
            
            if (is_dir(get_stylesheet_directory() .DS. 'gallery-from-files' .DS. $this->options['template'])) {
                $validTemplate = true;
                $this->getCSS(get_stylesheet_directory() .DS. 'gallery-from-files');
                $this->getJS(get_stylesheet_directory() .DS. 'gallery-from-files');
            } else {
                if (is_dir(GALLERY_FROM_FILES_595_ROOT .DS. 'templates' .DS. $this->options['template'])) {
                    $validTemplate = true;
                    $this->getCSS(GALLERY_FROM_FILES_595_ROOT .DS. 'templates');
                    $this->getJS(GALLERY_FROM_FILES_595_ROOT .DS. 'templates');
                }
            }

            // If template _FOLDER_ not found
            if (!$validTemplate) {
                
                $this->options['template'] = 'images';
                $validTemplate = true;
                
                $this->errorMessage[] = '<!-- Gallery from files: ' . 
                                        sprintf(
                                            __('Template folder not exists! <small>(%s)</small>', 'gallery-from-files'), 
                                            $this->options['template']) . 
                                        ' -->';
            } 

            if (is_writeable(GALLERY_FROM_FILES_595_WP_ROOT . $this->options['folder'])) {
                $this->is_writeable = true;
            } else {
                $this->errorMessage[] = '<!-- Gallery from files: ' . 
                                        sprintf(
                                            __('This folder not writeable! (%s)', 'gallery-from-files'), 
                                            $this->options['folder']) . 
                                        ' -->';
            }


			$ajax_vars = array(
                    'folder'     => $this->options['folder'],
					'order'      => $this->options['order'],
					'template'   => $this->options['template'],
					'th_size'    => $this->options['th_size'],
					'th_folder'  => $this->options['th_folder'],
					'page_limit' => $this->options['page_limit'],
					'title'      => $this->options['title'],
                    'token'      => $this->options['token'],
                    'compress'   => $this->options['compress'],
                    'webp'       => $this->options['webp']
				);

            $uniques = $this->getUniqueOptions($this->options);
            if (!empty($uniques)) {
                $customs = array();
                foreach ($uniques as $unique) {
                    $customs[$unique] = $this->options[$unique];
                }
                $ajax_vars = array_merge($ajax_vars, $customs);
            }
            
            // Javascript változók átadása
            // Set the javascript values
            if (count($ajax_vars) > 0) {
    			wp_register_script('gallery-from-files-'.$this->options['token'].'-variables-js', '', array('jquery'), false, true);
    			wp_localize_script('gallery-from-files-'.$this->options['token'].'-variables-js', 'gallery_from_files_variables_'.$this->options['token'], $ajax_vars);
    			wp_enqueue_script('gallery-from-files-'.$this->options['token'].'-variables-js');
            }
        }

        // Get user defined variables in shortcode
        private function getUniqueOptions()
        {
            $default = gallery_from_files_595_get_options();
            $default = array_keys($default);
            $default = array_merge($default, array('token', 'max', 'pagination', 'title', 'counter', 'image'));
            
            $custom = array_keys($this->options);
            $custom = array_diff($custom, $default);	
            $custom = array_values($custom);

            return $custom;	
        }
        
        // Replace user defined values from shortcode
        function prepareUniqueOptions($tmpl)
        {
            $uniques = $this->getUniqueOptions();
            foreach ($uniques as $unique) {
                $tmpl = str_replace('['.strtoupper($unique).']', $this->options[$unique], $tmpl);
            }
            return $tmpl;
        }

        private function getToken()
        {
            $chars = 'ABCDEFGHIJKLMNOPQURSTXYVWZ0123456789';
            $token = '';
            for ($i = 1; $i <= 10; $i++) {
            	$token .= substr($chars, rand(0, strlen($chars)-1), 1);
            }                
            return $token;
        }

        function prepareContent() {   
            $html = '';
            if (!empty($this->errorMessage)) {
                $html = "\r\n" . implode("\r\n", $this->errorMessage) . "\r\n";
                //return $html;
            }

            // If exsist "folder.html" in template folder --> one folder
            $image_html = $this->getTemplate('folder.html', true);
            
            if ($image_html != '') {
                $this->image_html = $image_html;
                $options_save = $this->options;
                
                $path = explode(DS, $this->options['folder']);
                $folder_name = $path[count($path)-1];
                unset($path[count($path)-1]);
                $this->options['folder'] = implode(DS, $path);
                
                $folder = null;
                $folders = $this->getFolders();
                
                if (!empty($folders)) { 
                    $index = array_search($folder_name, array_column($folders, 'filename'));
                    if ($index !== false) {
                        $folder = $folders[$index];
                    }                    
                }
                if ($folder != null) {
                    $html .= '<div class="gallery-from-files gallery-from-files-folders" data-token="'.$this->options['token'].'">'.
                             $this->renderFolders(array($folder)).
                             '</div>';
                }
                $this->options = $options_save;
            }

            // If exsist "folders.html" in template folder --> all folders
            $image_html = $this->getTemplate('folders.html', true);
            if ($image_html != '') {
                $this->image_html = $image_html;
                $folders = $this->getFolders();

                if (!empty($folders)) {

                    $count = count($folders); 
                    if ($this->options['page_limit'] > 0) {
                        $folders = array_slice($folders, 0, $this->options['page_limit']);
                    }
               
                    $html .= '<ul id="galley-folders-'.$this->options['token'].'" class="gallery-from-files gallery-from-files-folders" data-token="'.$this->options['token'].'">'.
                             $this->renderFolders($folders).
                             '<div style="clear:both"></div>' .
                             '</ul>' .
                             '<div style="clear:both"></div>' .
                             $this->getPagination($count, $this->options['page_limit'], 1, 'galley-folders-'.$this->options['token'], $this->options['token'], 'folders');
                }                
            }


            // If exsist "image.html" in template folder --> one image
            $image_html = $this->getTemplate('image.html', true);
            if ($image_html != '') {
                $options_save = $this->options;
                $this->image_html = $image_html;
                
                $image = null;
                if (is_file(GALLERY_FROM_FILES_595_WP_ROOT . $this->options['folder'])) {
                    $image_file = pathinfo($this->options['folder'], PATHINFO_BASENAME);
                    $this->options['folder'] = pathinfo($this->options['folder'], PATHINFO_DIRNAME);
                    $images = $this->getImages();
                    $index = array_search($image_file, array_column($images, 'filename'));
                    if ($index !== false) {
                        $image = $images[$index];
                    }                    
                } else {
                    $images = $this->getImages();
                    $image = null;       
                    if (!empty($images)) {

                        if (!empty($this->_image)) {
                            for ($i = 0; $i <= count($images)-1; $i++) {
                            	if ($images[$i]['filename'] == $this->_image) {
                            	   $image = $images[$i]; 
                            	}
                            }
                        } else {
                            if (!empty($this->_counter)) {
                                
                                if ($this->_counter < 0) {
                                    $this->_counter = (count($images) + $this->_counter) + 1;
                                }
                                
                                for ($i = 0; $i <= count($images)-1; $i++) {
                                	if ($i == ($this->_counter - 1)) {
                                	   $image = $images[$i]; 
                                	}
                                }
                            }                
                        }
                        if (empty($image)) {
                            $image  = $images[0];                    
                        }
                    }
                }
                if ($image != null) { 
                
                    $images = $this->checkImages(array($image));
                    $html .= '<div class="gallery-from-files gallery-from-files-images" data-token="'.$this->options['token'].'">' . 
                             $this->renderImages($images) .
                             '</div>';
                }
                         
                $this->options = $options_save;
            }


            // If exsist "images.html" in template folder --> all images in folder
            $image_html = $this->getTemplate('images.html', true);
            if ($image_html != '') {
                $this->image_html = $image_html;
                $images = $this->getImages();
                
                if (!empty($images)) {
                    $count = count($images); 
                    if ($this->options['page_limit'] > 0) {
                        $images = array_slice($images, 0, $this->options['page_limit']);
                    }
                
                    $images = $this->checkImages($images);
                    $html .= '<ul id="galley-images-'.$this->options['token'].'" class="gallery-from-files gallery-from-files-images" data-token="'.$this->options['token'].'">'.
                             $this->renderImages($images).
                             '<div style="clear:both"></div>' .
                             '</ul>' .
                             '<div style="clear:both"></div>' .
                             $this->getPagination($count, $this->options['page_limit'], 1, 'galley-images-'.$this->options['token'], $this->options['token'], 'images');

                } else {
                    $html .= "\r\n" . '<!-- Gallery from files: not found image. ('.$this->options['folder'].') -->' . "\r\n";
                }                
            }

            return $html;
        }

        function renderFolders($folders)
        {

            $html = array();
            if (empty($folders)) { return ''; }

            $size = explode('x', $this->options['th_size']);
            $url = esc_url( home_url( '/' ) );
            
            $this->image_html = str_replace('[TOKEN]',  $this->options['token'], $this->image_html);
            $this->image_html = str_replace('[WIDTH]',  $size[0], $this->image_html);
            $this->image_html = str_replace('[HEIGHT]', $size[1], $this->image_html);
            
            // A WebP-nek megfelelő bejegyzés kikeresése
            $regex = '/<picture>(\s*.*?)<\/picture>/s';
            preg_match( $regex, $this->image_html, $matches);
            if (isset($matches[0])) {
                $img_webp = $matches[0];

                $regex = '/<img[^>]+>/i';
                preg_match( $regex, $img_webp, $matches);
                if (isset($matches[0])) {
                    $img_default = $matches[0];
                }
            }

            $date_format = get_option( 'date_format' );

            foreach ($folders as $folder) {

                $tmpl = $this->image_html;
            
                $folder['id'] = 'gallery_'.$this->options['token'].'_'.$folder['alias'];
                $tmpl = str_replace('[ID]', $folder['id'], $tmpl);

                $tmpl = $this->conditionalTemplate($tmpl, $folder);
                
                if (!isset($folder['image']['webp'])) {
                    // Ha nincs WebP kép, akkor csak a kép bejegyzést hagyjuk meg.
                    $tmpl = str_replace($img_webp, $img_default, $tmpl);
                    $tmpl = str_replace('[WEBP]', '', $tmpl);                    
                    $tmpl = str_replace('[TH_WEBP]', '', $tmpl);
                } else {
                    $tmpl = str_replace('[WEBP]', $url . $this->options['folder'] .DS. $folder['filename'] .DS. $folder['image']['webp'], $tmpl);
                    $tmpl = str_replace('[TH_WEBP]', $url . $this->options['folder'] .DS. $folder['filename'] .DS. $this->options['th_folder'] .DS. $folder['image']['webp'], $tmpl);
                }
                
                if ($this->options['title']) {
                    if ($folder['title'] != $folder['filename']) {
                        $tmpl = str_replace('[TITLE]', $folder['title'], $tmpl);
                    } else {
                        $tmpl = str_replace('[TITLE]', '', $tmpl);
                    }
                } else {
                    $tmpl = str_replace('[TITLE]', '', $tmpl);
                }
                $tmpl = str_replace('[IMAGE]',     $url . $this->options['folder'] .DS. $folder['filename'] .DS. $folder['image']['filename'], $tmpl);
                $tmpl = str_replace('[TH_IMAGE]',  $url . $this->options['folder'] .DS. $folder['filename'] .DS. $folder['image']['thumbnail'], $tmpl);
                $tmpl = str_replace('[ALIAS]',     $folder['alias'], $tmpl);
                $tmpl = str_replace('[EXT]',       $folder['image']['ext'], $tmpl);

                $date_text = date_i18n( $date_format, strtotime($folder['date']));
                $tmpl = str_replace('[DATE]', $date_text, $tmpl);

                $tmpl = $this->prepareUniqueOptions($tmpl);
                
                $pos = strpos($tmpl, '[POPUP_IMAGES]');
                if ($pos !== false) {
                    $popup_tmpl = $this->getTemplate('popup-images.html');    
                    if ($popup_tmpl == '') {

                        $tmpl = str_replace('[POPUP_IMAGES]', '', $tmpl);

                    } else {

                        if (!class_exists ('JDovi42Galleries', false)) {
                            include_once(dirname(__file__) .DIRECTORY_SEPARATOR. 'jdovi42_galleries.php');
                        }
                        $files = new JDovi42Galleries(GALLERY_FROM_FILES_595_WP_ROOT . $this->options['folder'] .DS. $folder['filename']);
                        $files->allowExt = array('jpg', 'png', 'gif');
                        $this->setSortOrder($files);
                        $images = $files->getFiles();
                        
                        $tmpl = str_replace('[COUNT]', count($images), $tmpl);

                        // Az első képet töröljük, mert az eleve megjelenik a mappa megjelenítésénél.
                        if (!empty($images)) { unset($images[0]); }
                        
                        if (empty($images)) {
                            $tmpl = str_replace('[POPUP_IMAGES]', '', $tmpl);
                        } else {

                            $img_html = array();
                            foreach ($images as $image) {

                                $img_tmp = $popup_tmpl;

                                $img_tmp = str_replace('[TOKEN]',  $this->options['token'], $img_tmp);
                                $img_tmp = str_replace('[ID]',  $folder['id'], $img_tmp);
                                $img_tmp = str_replace('[IMAGE]',  $url . $this->options['folder'] .DS. $folder['filename'] .DS. $image['filename'], $img_tmp);
                                if (isset($image['thumbnail'])) {
                                    $img_tmp = str_replace('[TH_IMAGE]',  $url . $this->options['folder'] .DS. $folder['filename'] .DS. $image['thumbnail'], $img_tmp);
                                } else {
                                    $img_tmp = str_replace('[TH_IMAGE]', '', $img_tmp);
                                }

                                if ($this->options['title']) {
                                    if ($image['title'] != $image['filename']) {
                                        $img_tmp = str_replace('[TITLE]', $image['title'], $img_tmp);
                                    } else {
                                        $img_tmp = str_replace('[TITLE]', '', $img_tmp);
                                    }
                                } else {
                                    $img_tmp = str_replace('[TITLE]', '', $img_tmp);
                                }
                                
                                $img_tmp = $this->prepareUniqueOptions($img_tmp);
                                
                                $img_html[] = $img_tmp; 
                            }
                            $img_html = implode("\r\n", $img_html);
                            $tmpl = str_replace('[POPUP_IMAGES]', $img_html, $tmpl);
                        }
                    }
                }
                
                $html[] = $tmpl; 
            }            

            $html = implode("\r\n", $html);
            return $html;
        }

        private function conditionalTemplate($template, $data = array()) {
            
            // Ha nem kap adattartalmat, akkor az osztály adatait tartalmazza
            if (empty($data)) {
                $data = $this->options;
            }
            // Ha ez egy objektum, akkor tömböt csinálunk belőle.
            if (is_object($data)) { $data = (array) $data; }
            
            // Feltételekhez használt kapcsolók tömbje
            $conditionals = array();
            $regex = '/<!-- if (\s*.*?) -->/s';
            preg_match_all( $regex, $template, $matches);
            if (!empty($matches)) {
                foreach ($matches[1] as $name) {
                    if (!in_array($name, $conditionals)) {
                        $conditionals[] = $name;
                    }
                }
            }

            // Minden feltételhez kapcsolt változóra vizsgáljuk a sablont
            foreach ($conditionals as $conditional) {

                $regex = '/<!-- if '.$conditional.' -->(\s*.*?)<!-- endif '.$conditional.' -->/s';
                preg_match_all( $regex, $template, $matches);
                if (!empty($matches)) {
                    for ($i = 0; $i <= count($matches[0])-1; $i++) {
                        $content = explode('<!-- else '.$conditional.' -->', $matches[1][$i]);
                        if (isset($data[$conditional]) and !empty($data[$conditional])) {
                            $template = str_replace($matches[0][$i], $content[0], $template);
                        } else {
                            if (isset($content[1])) {
                                $template = str_replace($matches[0][$i], $content[1], $template);
                            } else {
                                $template = str_replace($matches[0][$i], '', $template);
                            }
                        }
                    }
                }
            } 
            return $template;           
        }


        function renderImages($images) {

            $html = array();
            
            if (empty($images)) { return ''; }
            
            $size = explode('x', $this->options['th_size']);
            $url = esc_url( home_url( '/' ) );
            
            $token = $this->getToken();
            
            $id = 'images_'.$token;
            $this->image_html = str_replace('[ID]', $id, $this->image_html);

            $this->image_html = str_replace('[TOKEN]',  $this->options['token'], $this->image_html);
            $this->image_html = str_replace('[WIDTH]',  $size[0], $this->image_html);
            $this->image_html = str_replace('[HEIGHT]', $size[1], $this->image_html);

            // A WebP-nek megfelelő bejegyzés kikeresése
            $regex = '/<picture>(\s*.*?)<\/picture>/s';
            preg_match( $regex, $this->image_html, $matches);
            if (isset($matches[0])) {
                $img_webp = $matches[0];

                $regex = '/<img[^>]+>/i';
                preg_match( $regex, $img_webp, $matches);
                if (isset($matches[0])) {
                    $img_default = $matches[0];
                }
            }
            
            if (!class_exists ('JDovi42Files', false)) {
                include_once(dirname(__file__) .DIRECTORY_SEPARATOR. 'jdovi42_files.php');
            }
            $source  = new JDovi42Files(GALLERY_FROM_FILES_595_WP_ROOT . $this->options['folder']);
            $date_format = get_option( 'date_format' );

            foreach ($images as $image) {

                $tmpl = $this->image_html;
                $tmpl = $this->conditionalTemplate($tmpl, $image);                

                if (!isset($image['webp'])) {
                    // Ha nincs WebP kép, akkor csak a kép bejegyzést hagyjuk meg.
                    $tmpl = str_replace($img_webp, $img_default, $tmpl);
                    $tmpl = str_replace('[WEBP]', '', $tmpl);                    
                    $tmpl = str_replace('[TH_WEBP]', '', $tmpl);
                } else {
                    $tmpl = str_replace('[WEBP]', $url . $this->options['folder'] .DS. $image['webp'], $tmpl);
                    $tmpl = str_replace('[TH_WEBP]', $url . $this->options['folder'] .DS. $this->options['th_folder'] .DS. $image['webp'], $tmpl);
                }
               
                if ($this->options['title']) {
                    if ($image['title'] != $image['filename']) {
                        $tmpl = str_replace('[TITLE]', $image['title'], $tmpl);
                    } else {
                        $tmpl = str_replace('[TITLE]', '', $tmpl);
                    }
                } else {
                    $tmpl = str_replace('[TITLE]', '', $tmpl);
                }
                
                $tmpl = str_replace('[ID]',        $id, $tmpl);
                $tmpl = str_replace('[IMAGE]',     $url . $this->options['folder'] .DS. $image['filename'], $tmpl);
                $tmpl = str_replace('[TH_IMAGE]',  $url . $this->options['folder'] .DS. $image['thumbnail'], $tmpl);
                $tmpl = str_replace('[ALIAS]',     $image['alias'], $tmpl);
                $tmpl = str_replace('[EXT]',       $image['ext'], $tmpl);
                $tmpl = str_replace('[SIZE]',      $source->filesize_format($image['size']), $tmpl);

                $date_text = date_i18n( $date_format, strtotime($image['date']));
                $tmpl = str_replace('[DATE]', $date_text, $tmpl);
                
                $tmpl = $this->prepareUniqueOptions($tmpl);
                
                $html[] = $tmpl; 
            }            
            
            $html = implode("\r\n", $html);
            return $html;
        }


        function checkImages($images)
        {
            for ($i = 0; $i <= count($images)-1; $i++) {
            	$images[$i] = $this->checkOneImage($images[$i], $this->options['folder']);
            }
            return $images;
        }

        // Check thumbnail and webp image
        function checkOneImage($image, $folder) {

            if (!class_exists ('JDovi42Image', false)) {
                include_once(dirname(__file__) .DIRECTORY_SEPARATOR. 'jdovi42_image.php');
            }

            if (!isset($image['thumbnail'])) {

                // This image new upload

                $source = new JDovi42Image(GALLERY_FROM_FILES_595_WP_ROOT . $folder .DS. $image['filename']);
                $source->jpeg_compress = $this->options['compress'];
                
                if (is_writeable(GALLERY_FROM_FILES_595_WP_ROOT.$folder)) {

                    if ($this->options['auto'] == 1) {
                        $source->resizeImage($this->options['size']);
                    }
                    $source->createThumb($this->options['th_size'], $this->options['th_folder']);
                    $image['thumbnail'] = $this->options['th_folder'] .DS. $image['filename'];
                    if ($this->options['webp'] == 1) {
                        $source->createWebP($this->options['th_folder']);
                    }
                    if (($this->options['auto'] == 1) and ($this->options['watermark'] != '')) {
                        $source->watermark['walign'] = 'right';
                        $source->watermark['halign'] = 'bottom';
                        $source->createWatermark($this->options['watermark']);
                    }
                    if ($this->options['webp'] == 1) {
                        $source->createWebP();
                        $image['webp'] = pathinfo($image['filename'],  PATHINFO_FILENAME).'.webp';
                    }
                } else {
                    $image['thumbnail'] = $image['filename'];
                }
                
                
            } else {
                
                // Check and generate webp images
                if (($this->options['webp'] == 1) and ($this->is_writeable)) {

                    if (file_exists(GALLERY_FROM_FILES_595_WP_ROOT . $folder .DS. pathinfo($image['filename'],  PATHINFO_FILENAME).'.webp')) {
                        $image['webp'] = pathinfo($image['filename'],  PATHINFO_FILENAME).'.webp';                           
                    } else {
                        $source = new JDovi42Image(GALLERY_FROM_FILES_595_WP_ROOT . $folder .DS. $image['filename']);
                        $source->jpeg_compress = $this->options['compress'];
                        $source->createWebP();
                        
                        $image['webp'] = pathinfo($image['filename'],  PATHINFO_FILENAME).'.webp';
                    }    
                    if (!file_exists(GALLERY_FROM_FILES_595_WP_ROOT . $folder .DS. $this->options['th_folder'] .DS. pathinfo($image['filename'],  PATHINFO_FILENAME).'.webp')) {
                        $source = new JDovi42Image(GALLERY_FROM_FILES_595_WP_ROOT . $folder .DS. $this->options['th_folder'] .DS. $image['filename']);
                        $source->jpeg_compress = $this->options['compress'];
                        $source->createWebP();
                    }    

                    $image['webp'] = pathinfo($image['filename'],  PATHINFO_FILENAME).'.webp';

                }
            }
               
            return $image;            
        }

        function getImages() {

            if (is_file(GALLERY_FROM_FILES_595_WP_ROOT . $this->options['folder'])) {
                return array();                
            }
            if (!class_exists ('JDovi42Galleries', false)) {
                include_once(dirname(__file__) .DIRECTORY_SEPARATOR. 'jdovi42_galleries.php');
            }
            $files = new JDovi42Galleries(GALLERY_FROM_FILES_595_WP_ROOT . $this->options['folder'], $this->options['th_folder']);
            //$files->allowExt = array('jpeg', 'jpg', 'png', 'gif');
            $this->setSortOrder($files);
            $images = $files->getFiles();

            if (!empty($this->_image)) {
                $temp = array();
                for ($i = 0; $i <= count($images)-1; $i++) {
                	if ($images[$i]['filename'] == $this->_image) {
                	   $temp[] = $images[$i]; 
                	}
                }
                $images = $temp;
            } else {
                if (!empty($this->_counter)) {
                    $temp = array();
                    for ($i = 0; $i <= count($images)-1; $i++) {
                        if ($this->_counter < 0) {
                            $this->_counter = (count($images) + $this->_counter) + 1;
                        }
                    	if ($i == ($this->_counter - 1)) {
                    	   $temp[] = $images[$i]; 
                    	}
                    }
                    $images = $temp;
                }                
            }

            
            if (isset($this->options['max'])) {
                $images = array_slice($images, 0, $this->options['max']);
            }
            
            return $images;            
        }

        
        function getPagination($count, $limit, $actual, $id, $token, $type)
        {
            $html = '';
            if ((isset($this->options['pagination'])) and ($this->options['pagination'] == 0)) {
                return '';
            }
            if ((isset($this->options['page_limit'])) and ($this->options['page_limit'] < 1)) {
                return '';
            }
            if ($count > $limit) {
                $pages = ceil($count / $limit);
                
                $html .= '<ul class="gallery-from-files-pagination" data-id="'.$id.'" data-token="'.$token.'" data-type="'.$type.'"  data-actual="1">' . "\r\n";
                for ($i = 1; $i <= $pages; $i++) {
                    if ($i == $actual) {
                        $current = 'current';
                    } else {
                        $current = '';
                    }
                    $html .= '<li class="gallery-from-files-page-item"><span class="gallery-from-files-page-link '.$current.'" onclick="gallery_from_files_595_get_page(\''.$i.'\', jQuery(this));">'.$i.'</span></li>' . "\r\n";
                }
                $html .= '</ul>' . "\r\n" . 
                         '<div style="clear:both;"></div>' . "\r\n";
            }
            return $html;            
        }

        function getFolders()
        {
            if (is_file(GALLERY_FROM_FILES_595_WP_ROOT . $this->options['folder'])) {
                return array();                
            }
            if (!class_exists ('JDovi42Galleries', false)) {
                include_once(dirname(__file__) .DIRECTORY_SEPARATOR. 'jdovi42_galleries.php');
            }
            $source = new JDovi42Galleries(GALLERY_FROM_FILES_595_WP_ROOT . $this->options['folder'], $this->options['th_folder']);
            $this->setSortOrder($source);
            $folders = $source->getFolders();
            
            if (!empty($folders)) {
                
                $is_empty = array();
                                
                for ($i = 0; $i <= count($folders)-1; $i++) {
                	
                    $files = new JDovi42Galleries(GALLERY_FROM_FILES_595_WP_ROOT . $this->options['folder'] .DS. $folders[$i]['filename'], $this->options['th_folder']);
                    $this->setSortOrder($files);
                    $files->allowExt = array('jpg', 'png', 'gif');
                    
                    $images = $files->getFiles();
                    if (!empty($images)) {
                        
                        if (!empty($this->_image)) {
                            for ($j = 0; $j <= count($images)-1; $j++) {
                            	if ($images[$j]['filename'] == $this->_image) {
                                    $images[$j] = $this->checkOneImage($images[$j], $this->options['folder'] .DS. $folders[$i]['filename']);
                                    $folders[$i]['image'] = $images[$j];
                            	}
                            }
                        } else {
                            if (!empty($this->_counter)) {
                                for ($j = 0; $j <= count($images)-1; $j++) {
                                    if ($this->_counter < 0) {
                                        $this->_counter = (count($images) + $this->_counter) + 1;
                                    }
                                    if ($j == ($this->_counter - 1)) {
                                        $images[$j] = $this->checkOneImage($images[$j], $this->options['folder'] .DS. $folders[$i]['filename']);
                                        $folders[$i]['image'] = $images[$j];
                                	}
                                }
                            }                
                        }
                        if (!isset($folders[$i]['image'])) {
                            $images[0] = $this->checkOneImage($images[0], $this->options['folder'] .DS. $folders[$i]['filename']);
                            $folders[$i]['image'] = $images[0];
                        }
                    } else {
                        $is_empty[] = $i;
                    }
                }
                if (!empty($is_empty)) {
                    $result = array();
                    foreach ($folders as $key => $value) {
                        if (!in_array($key, $is_empty)) {
                            $result[] = $value;
                        }
                    }
                    $folders = $result;
                }
            }
            return $folders;
            
        }

        function setSortOrder($source)
        {
            switch ($this->options['order']){ 
            	case 'date':
            	   $source->folderSort = 'sortByDate';
            	   $source->filesSort = 'sortByDate';
                   break;
            
            	case 'name':
            	   $source->folderSort = 'sortByName';
            	   $source->filesSort = 'sortByName';
                   break;
            
            	default :
            	   $source->folderSort = 'sortByList';
            	   $source->filesSort = 'sortByList';
            }
            switch ($this->options['orderby']){ 
            	case 'desc':
                   $source->filesSortOrder = 'desc';
                   break; 
            	default :
                   $source->filesSortOrder = 'asc';
             }
        }

        function getTemplate($filename, $silent = false) {

            $template = '';
            if (file_exists(get_stylesheet_directory() .DS. 'gallery-from-files' .DS. $this->options['template'] .DS. $filename)) {
                $template = file_get_contents(get_stylesheet_directory() .DS. 'gallery-from-files' .DS. $this->options['template'] .DS. $filename);
            } else {
                if (file_exists(GALLERY_FROM_FILES_595_ROOT .DS. 'templates' .DS. $this->options['template'] .DS. $filename)) {
                    $template = file_get_contents(GALLERY_FROM_FILES_595_ROOT .DS. 'templates' .DS. $this->options['template'] .DS. $filename);
                } else {
                    if (!$silent) {
                        echo $this->showError(
                                __('Template file not exists! <small>(%s)</small>', 'gallery-from-files'),
                                $filename
                            );
                    }
                    return '';
                }
            }
            return $template;
        }
        
        function getCSS($source) {

            // CSS fájlok beolvasása, ha vannak
            if (!class_exists ('JDovi42Files', false)) {
                include_once(dirname(__file__) .DS. 'jdovi42_files.php');
            }
            $files = new JDovi42Files($source .DS. $this->options['template']);
            $files->allowExt = array('css');
            $css = $files->getFiles();
            
            if (!empty($css)) {
                $url = esc_url( home_url( '/' ) );
                $url = $url . str_replace(GALLERY_FROM_FILES_595_WP_ROOT, '', $source) .'/'. $this->options['template'];
                $url = str_replace(DS, '/', $url);
                for ($i = 0; $i <= count($css)-1; $i++) {
                    wp_register_style( 'massage-gallery-style-'.$this->options['template'].'-'.$i, $url .'/'. $css[$i]['filename'], array(), GALLERY_FROM_FILES_595_VERSION );
                	wp_enqueue_style(  'massage-gallery-style-'.$this->options['template'].'-'.$i );
                }
            }
            
        }

        function getJS($source) {
            if (!class_exists ('JDovi42Files', false)) {
                include_once(dirname(__file__) .DS. 'jdovi42_files.php');
            }
            $files = new JDovi42Files($source .DS. $this->options['template']);
            $files->allowExt = array('js');
            $js = $files->getFiles();
            if (!empty($js)) {
                $url = esc_url( home_url( '/' ) );
                $url = $url . str_replace(GALLERY_FROM_FILES_595_WP_ROOT, '', $source) .'/'. $this->options['template'];
                $url = str_replace(DS, '/', $url);
                for ($i = 0; $i <= count($js)-1; $i++) {
        			wp_register_script('massage-gallery-js-'.$this->options['template'].'-'.$i, $url .'/'. $js[$i]['filename'], array('jquery'), GALLERY_FROM_FILES_595_VERSION );  
        			wp_enqueue_script('massage-gallery-js-'.$this->options['template'].'-'.$i);
                }
            }
        }
        
        function showError($text, $filename) {
            echo 
                '<div class="gallery-from-files-warning">' . "\r\n" .
                 sprintf( $text, $filename) . "\r\n" .
                 '</div>' . "\r\n";
        }
    }