<?php

if ( !defined( 'ABSPATH' ) ) exit;

class gallery_from_files_595_settings {
    private $options;
    private $webp = false; // ? imagewebp()

    public function __construct()
    {
        add_action( 'admin_menu', array( $this, 'add_plugin_page' ) );
        add_action( 'admin_init', array( $this, 'page_init' ) );
        
        if (function_exists('imagewebp')) {
            $this->webp = true;
        }
    }

    public function add_plugin_page()
    {
        // This page will be under "Settings"
        add_options_page(
            'Settings Admin', 
            __('Gallery from files settings', 'gallery-from-files'), 
            'manage_options', 
            'gallery-from-files-setting-admin', 
            array( $this, 'create_admin_page' )
        );
    }

    public function create_admin_page()
    {
        $this->options = gallery_from_files_595_get_options ( 'gallery_from_files_options' );
        ?>
        <div class="wrap">
            <h1><?php echo __('Gallery from files settings', 'gallery-from-files');?></h1>
            <?php
                $active_tab = isset( $_GET[ 'gallery-from-files-tab' ] ) ? sanitize_text_field($_GET[ 'gallery-from-files-tab' ]) : 'galleries';
            ?>
            <h2 class="nav-tab-wrapper">
                <a href="?page=gallery-from-files-setting-admin&gallery-from-files-tab=galleries" class="nav-tab  <?php echo $active_tab == 'galleries' ? 'nav-tab-active' : ''; ?>"><?php echo __('Galleries', 'gallery-from-files');?></a>
                <a href="?page=gallery-from-files-setting-admin&gallery-from-files-tab=settings" class="nav-tab  <?php echo $active_tab == 'settings' ? 'nav-tab-active' : ''; ?>"><?php echo __('Plugin settings', 'gallery-from-files');?></a>
                <a href="?page=gallery-from-files-setting-admin&gallery-from-files-tab=use" class="nav-tab  <?php echo $active_tab == 'readme' ? 'nav-tab-active' : ''; ?>"><?php echo __('Use gallery', 'gallery-from-files');?></a>
                <a href="?page=gallery-from-files-setting-admin&gallery-from-files-tab=import" class="nav-tab  <?php echo $active_tab == 'import' ? 'nav-tab-active' : ''; ?>"><?php echo __('Import gallery', 'gallery-from-files');?></a>
            </h2>

            <?php
                switch ($active_tab){ 

                    case 'settings':
                        ?>
                        <form method="post" action="options.php">
                        <?php
                            // This prints out all hidden setting fields
                            settings_fields( 'gallery_from_files_option_main' );
                            do_settings_sections( 'gallery-from-files-setting-admin' );
                            submit_button();
                        ?>
                        </form>
                        <?php
                        exit;

                    case 'use':
                        echo '<h3>' . __('Using title', 'gallery-from-files'). '</h3>';
                        include( dirname(__FILE__) . DS . 'how_to_use.php' );
                        exit;

                    case 'import':
                        include( dirname(__FILE__) . DS . 'import.php' );
                        exit;

                    
                    default: 
                        include( dirname(__FILE__) . DS . 'gallery.php' );
                        $options = gallery_from_files_595_get_options();
                        $size = explode('x', $options['th_size']);
                        /*
                        if ($size[0] > 200) {
                            $options['th_size'] = '200x200';
                        }
                        */
                        if (isset($_REQUEST['order'])) {
                            $order = sanitize_text_field($_REQUEST['order']);
                            $options['order'] = $order;
                        }
                        if (isset($_REQUEST['orderby'])) {
                            $orderby = sanitize_text_field($_REQUEST['orderby']);
                            $options['orderby'] = $orderby;
                        }
                        $gallery = new JGalleryFromFiles($options);
                        
                        $gallery->render();

                }
            ?>

        </div>
        <?php
    }

    /**
     * Register and add settings
     */
    public function page_init()
    {        
        register_setting(
            'gallery_from_files_option_main', // Option group
            'gallery_from_files_options', // Option name
            array( $this, 'sanitize' ) // Sanitize
        );

        add_settings_section(
            'gallery_from_files_settings_main', // ID
            __('Gallery from files Custom Settings', 'gallery-from-files'), // Title
            array( $this, 'print_section_info' ), // Callback
            'gallery-from-files-setting-admin' // Page
        );  

        add_settings_field(
            'size', // ID
            __('Image size', 'gallery-from-files'), // Title
            array( $this, 'size_callback' ), // Callback
            'gallery-from-files-setting-admin', // Page
            'gallery_from_files_settings_main' // Section
        );

        add_settings_field(
            'auto', // ID
            __('Auto correct', 'gallery-from-files'), // Title
            array( $this, 'auto_callback' ), // Callback
            'gallery-from-files-setting-admin', // Page
            'gallery_from_files_settings_main' // Section
        );

        add_settings_field(
            'rename', // ID
            __('File rename', 'gallery-from-files'), // Title
            array( $this, 'rename_callback' ), // Callback
            'gallery-from-files-setting-admin', // Page
            'gallery_from_files_settings_main' // Section
        );

        add_settings_field(
            'th_size', // ID
            __('Thumbnail size', 'gallery-from-files'), // Title
            array( $this, 'th_size_callback' ), // Callback
            'gallery-from-files-setting-admin', // Page
            'gallery_from_files_settings_main' // Section
        );

        add_settings_field(
            'th_folder', // ID
            __('Thumbnail folder', 'gallery-from-files'), // Title
            array( $this, 'th_folder_callback' ), // Callback
            'gallery-from-files-setting-admin', // Page
            'gallery_from_files_settings_main' // Section
        );

        add_settings_field(
            'page_limit', // ID
            __('Page limit', 'gallery-from-files'), // Title
            array( $this, 'page_limit_callback' ), // Callback
            'gallery-from-files-setting-admin', // Page
            'gallery_from_files_settings_main' // Section
        );

        add_settings_field(
            'order', // ID
            __('Order', 'gallery-from-files'), // Title
            array( $this, 'order_callback' ), // Callback
            'gallery-from-files-setting-admin', // Page
            'gallery_from_files_settings_main' // Section
        );

        add_settings_field(
            'fancy', // ID
            __('Load Fancybox', 'gallery-from-files'), // Title
            array( $this, 'fancy_callback' ), // Callback
            'gallery-from-files-setting-admin', // Page
            'gallery_from_files_settings_main' // Section
        );

        add_settings_field(
            'title', // ID
            __('Use title', 'gallery-from-files'), // Title
            array( $this, 'title_callback' ), // Callback
            'gallery-from-files-setting-admin', // Page
            'gallery_from_files_settings_main' // Section
        );

        add_settings_field(
            'template', // ID
            __('Default template', 'gallery-from-files'), // Title
            array( $this, 'template_callback' ), // Callback
            'gallery-from-files-setting-admin', // Page
            'gallery_from_files_settings_main' // Section
        );

        add_settings_field(
            'watermark', // ID
            __('Watermark image', 'gallery-from-files'), // Title
            array( $this, 'watermark_callback' ), // Callback
            'gallery-from-files-setting-admin', // Page
            'gallery_from_files_settings_main' // Section
        );

        add_settings_field(
            'webp', // ID
            __('WebP support', 'gallery-from-files'), // Title
            array( $this, 'webp_callback' ), // Callback
            'gallery-from-files-setting-admin', // Page
            'gallery_from_files_settings_main' // Section
        );

        add_settings_field(
            'compress', // ID
            __('Compression', 'gallery-from-files'), // Title
            array( $this, 'compress_callback' ), // Callback
            'gallery-from-files-setting-admin', // Page
            'gallery_from_files_settings_main' // Section
        );


    }

    public function sanitize( $input )
    {
        $new_input = array();

        if ( isset( $input['size'] ) ) {
            $tmp = preg_replace("/[^0-9x]/", "", $input['size']);
            $tmp = explode('x', $tmp);
            if ((!is_numeric($tmp[0]) or ($tmp[0] == '')) or (!is_numeric($tmp[1]) or ($tmp[1] == ''))) {
                $new_input['size'] = '800x600';
            } else {
                $new_input['size'] = $tmp[0] . 'x' . $tmp[1];
            }
        } else {
            $new_input['size'] = '800x600';
        }
        if ( isset( $input['auto'] ) ) {
            $new_input['auto'] = 1;
        } else {
            $new_input['auto'] = 0;
        }
        if ( isset( $input['rename'] ) ) {
            $new_input['rename'] = 1;
        } else {
            $new_input['rename'] = 0;
        }
        if ( isset( $input['th_size'] ) ) {
            $tmp = preg_replace("/[^0-9x]/", "", $input['th_size']);
            $tmp = explode('x', $tmp);
            if ((!is_numeric($tmp[0]) or ($tmp[0] == '')) or (!is_numeric($tmp[1]) or ($tmp[1] == ''))) {
                $new_input['th_size'] = '200x200';
            } else {
                $new_input['th_size'] = $tmp[0] . 'x' . $tmp[1];
            }
        } else {
            $new_input['th_size'] = '200x200';
        }
        if ( isset( $input['th_folder'] ) ) {
            $new_input['th_folder'] = trim( $input['th_folder'] );
        }
        if ( isset( $input['page_limit'] ) ) {
            $new_input['page_limit'] = trim( $input['page_limit'] );
        }
        if ( isset( $input['order'] ) ) {
            $new_input['order'] = trim( $input['order'] );
        }
        if ( isset( $input['fancy'] ) ) {
            $new_input['fancy'] = 1;
        } else {
            $new_input['fancy'] = 0;
        }
        if ( isset( $input['title'] ) ) {
            $new_input['title'] = 1;
        } else {
            $new_input['title'] = 0;
        }
        if ( isset( $input['template'] ) ) {
            $new_input['template'] = trim( $input['template'] );
        }
        if ( isset( $input['watermark'] ) ) {
            $new_input['watermark'] = trim( $input['watermark'] );
        }
        if ($this->webp) {
            if ( isset( $input['webp'] ) ) {
                $new_input['webp'] = 1;
            } else {
                $new_input['webp'] = 0;
            }
        } else {
            $new_input['webp'] = 0;
        }

        if ( isset( $input['compress'] ) ) {
            $new_input['compress'] = trim( $input['compress'] );
        }

        return $new_input;
    }

    public function print_section_info()
    {
        print __('You can overwrite some parameters when inviting!', 'gallery-from-files');
    }

    public function size_callback()
    {
        printf(
            '<input type="text" name="gallery_from_files_options[size]" value="%s" />' .
            '<br /><small>'.__('If the image is larger, it will be resized to this size.<br />Example: 800x600', 'gallery-from-files').'</small>' .
            '<br /><small>'.__('You will consider whether the image is portrait or landscape.', 'gallery-from-files').'</small>',
            isset( $this->options['size'] ) ? esc_attr( $this->options['size']) : ''
        );
    }

    public function auto_callback()
    {
        if ($this->options['auto'] == 1) {
            $checked = ' CHECKED ';
        } else {
            $checked = '';
        } 
        printf(
            '<input type="checkbox" name="gallery_from_files_options[auto]" value="1" '.$checked.' />' .
            '<br /><small>'.__('Automatic resizing and watermark if needed.', 'gallery-from-files').'</small>' . 
            '<br /><small>'.__('If, for example, You uploaded the image via FTP, then no thumbnail. In this case, it not only produces the thumbnail but also resizes the image and places the watermark on it.', 'gallery-from-files').'</small>',
            isset( $this->options['auto'] ) ? esc_attr( $this->options['auto']) : ''
        );
    }

    public function rename_callback()
    {
        if ($this->options['rename'] == 1) {
            $checked = ' CHECKED ';
        } else {
            $checked = '';
        } 
        printf(
            '<input type="checkbox" name="gallery_from_files_options[rename]" value="1" '.$checked.' />' .
            '<br /><small>'.__('If you turn it on, changing the description will also change the file name.', 'gallery-from-files').'</small>',
            isset( $this->options['rename'] ) ? esc_attr( $this->options['rename']) : ''
        );
    }

    public function th_size_callback()
    {
        printf(
            '<input type="text" name="gallery_from_files_options[th_size]" value="%s" />' .
            '<br /><small>'.__('Set the thumbnail size.<br />Example: 200x200 ', 'gallery-from-files').'</small>',
            isset( $this->options['th_size'] ) ? esc_attr( $this->options['th_size']) : ''
        );
    }

    public function th_folder_callback()
    {
        printf(
            '<input type="text" name="gallery_from_files_options[th_folder]" value="%s" />' .
            '<br /><small>'.__('Thumbnail folder name.<br />It will create it in each folder where it generates thumbnails.', 'gallery-from-files').'</small>',
            isset( $this->options['th_folder'] ) ? esc_attr( $this->options['th_folder']) : ''
        );
    }

    public function page_limit_callback()
    {
        printf(
            '<input type="number" name="gallery_from_files_options[page_limit]" value="%s"  min="0" max="9999" style="width:80px;" />' .
            '<br /><small>'.__('The number of images displayed per page.<br /> If zero, all images are displayed.', 'gallery-from-files').'</small>',
            isset( $this->options['page_limit'] ) ? esc_attr( $this->options['page_limit']) : ''
        );
    }

    public function order_callback()
    {
        echo '<!-- ' . __('set_title', 'gallery-from-files') . ' -->';
        echo '<!-- ' . __('name_title', 'gallery-from-files') . ' -->';
        echo '<!-- ' . __('date_title', 'gallery-from-files') . ' -->';
        $values = array('set', 'name', 'date');
        $html = '<select name="gallery_from_files_options[order]">';
        foreach ($values as $item) {
            $html .= '<option value="'.$item.'"';
            if ($this->options['order'] == $item) {
               $html .= ' SELECTED';
            }
            $html .= '>'.__($item.'_title', 'gallery-from-files').'</option>';
        }
        $html .= '</select>';
        $html .= '<br /><small>'.__('How to sort pictures.', 'gallery-from-files').'</small>';
        echo $html;
    }

    public function fancy_callback()
    {
        if ($this->options['fancy'] == 1) {
            $checked = ' CHECKED ';
        } else {
            $checked = '';
        } 
        printf(
            '<input type="checkbox" name="gallery_from_files_options[fancy]" value="1" '.$checked.' />' .
            '<br /><small>'.__('Load Fancybox in frontend. It will be loaded in the admin interface.', 'gallery-from-files').'</small>',
            isset( $this->options['fancy'] ) ? esc_attr( $this->options['fancy']) : ''
        );
    }

    public function title_callback()
    {
        if ($this->options['title'] == 1) {
            $checked = ' CHECKED ';
        } else {
            $checked = '';
        } 
        printf(
            '<input type="checkbox" name="gallery_from_files_options[title]" value="1" '.$checked.' />' .
            '<br /><small>'.__('If there is a description of the image, it will be displayed.', 'gallery-from-files').'</small>',
            isset( $this->options['title'] ) ? esc_attr( $this->options['title']) : ''
        );
    }

    public function template_callback()
    {
        if (!class_exists('JDovi42Files')) {
            include_once( dirname(__file__) .DS. 'jdovi42_files.php');
        }
        $files = new JDovi42Files(GALLERY_FROM_FILES_595_ROOT .DS. 'templates'); 
        $files->addTilt('sample');
        $files->folderSort = 'sortByName';
        $tmp = $files->getFolders();
        
        $templates = array();
        foreach ($tmp as $item) {
            $templates[$item['filename']] = $item['title'];
        }
        
        if (is_dir(get_stylesheet_directory() .DS. 'gallery-from-files')) {
            $files = new JDovi42Files(get_stylesheet_directory() .DS. 'gallery-from-files');
            $files->folderSort = 'sortByName';
            $in_themes = $files->getFolders();
            foreach ($in_themes as $theme) {
                if (!in_array($theme['filename'], $templates)) {
                    $templates[$theme['filename']] = $theme['title'];
                }
            }        
        }
        $selected = isset( $this->options['template'] ) ? esc_attr( $this->options['template']) : '';

        echo '<select name="gallery_from_files_options[template]">';
        foreach ($templates as $key => $value) {
            echo '<option value="'.$key.'" '.selected( $selected, $key ).'>'.$value.'</option>';
        }        
        echo '</select><br />' . 
             '<small>'.__('Select default template folder', 'gallery-from-files').'</small><br />' .
             '<small>'.sprintf(__('You can create your own templates in the %s folder', 'gallery-from-files'), str_replace(GALLERY_FROM_FILES_595_WP_ROOT, '', get_stylesheet_directory() .DS. 'gallery-from-files' .DS. '*')).'</small>';
    }

    public function watermark_callback()
    {
        if (!class_exists('JDovi42Files')) {
            include_once( dirname(__file__) .DS. 'jdovi42_files.php');
        }
        $upload = wp_get_upload_dir();
        $files = new JDovi42Files($upload['basedir']);
        $files->allowExt = array('jpg', 'png', 'gif');
        $images = $files->getFiles();
        $selected = isset( $this->options['watermark'] ) ? esc_attr( $this->options['watermark']) : '';
        
        echo '<select name="gallery_from_files_options[watermark]">';
        echo '<option value="" '.selected( $selected, $image['filename'] ).'>'.__('Not used', 'gallery-from-files').'</option>';
        foreach ($images as $image) {
            echo '<option value="'.$image['filename'].'" '.selected( $selected, $image['filename'] ).'>'.$image['filename'].'</option>';
        }        
        echo '</select>' . 
             '<br /><small>'.sprintf( __('Watermark image. <br /><small>(From: %s...) </small>', 'gallery-from-files'), str_replace(GALLERY_FROM_FILES_595_WP_ROOT, '', $upload['basedir'])).'</small>';   
    }

    public function webp_callback()
    {
        if ($this->webp) {
            if ($this->options['webp'] == 1) {
                $checked = ' CHECKED ';
            } else {
                $checked = '';
            } 
            printf(
                '<input type="checkbox" name="gallery_from_files_options[webp]" value="1" '.$checked.' />' .
                '<br /><small>'.__('Generate WebP images', 'gallery-from-files').'</small><br />'.
                '<small>'.__('This feature duplicates images because they are also created in webp format.', 'gallery-from-files').'</small>',
                isset( $this->options['webp'] ) ? esc_attr( $this->options['webp']) : ''
            );
        } else {
            echo __('This system not supported imagewebp() function.', 'gallery-from-files');
            echo '<input type="hidden" name="gallery_from_files_options[webp]" value="0" />';
        }
    }

    public function compress_callback()
    {
        printf(
            '<input type="number" name="gallery_from_files_options[compress]" value="%s"  min="1" max="100" />' .
            '<br /><small>'.__('JPG and WebP compression rate.', 'gallery-from-files').'</small>',
            isset( $this->options['compress'] ) ? esc_attr( $this->options['compress']) : ''
        );
    }

}
